"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GraphQlGenDrivers = void 0;

var _utils = require("@graphql-tools/utils");

var _graphql = require("graphql");

var _https = _interopRequireDefault(require("https"));

var _http = _interopRequireDefault(require("http"));

var _tsJsonObject = require("ts-json-object");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let pluginName = `GraphQL Query Generator`;

function chunk(arr, size) {
  let chunks = [];
  let i = 0;

  while (i < arr.length) {
    chunks.push(arr.slice(i, i += size));
  }

  return chunks;
} //*******
//******* Drivers
//*******


class GraphQlGenDrivers {
  async generateGraphQlFromSchema(context, models, folderName, baseUrl, importSchema) {
    try {
      if (importSchema) {
        let schema = await buildGraphQLSchema(importSchema);
        let operations = await generateOperations(schema);
        await importToCurrentWorkspace(models, operations, baseUrl, folderName, context);
        context.app.alert(`${pluginName}: Import from schema`, "Successfully imported GraphQL operations from schema");
      }
    } catch (error) {
      if (error instanceof Error) {
        console.log(`[ERROR] [${pluginName}: From schema] [${error}]`);
        context.app.alert("Error while importing GraphQL operations", error.message);
      }
    }
  }

  async generateGraphQlFromUrl(context, models, folderName, baseUrl, importUrl) {
    try {
      if (importUrl) {
        let schema = await fetchGraphQLSchema(importUrl, context);
        let operations = await generateOperations(schema);
        await importToCurrentWorkspace(models, operations, baseUrl, folderName, context);
        context.app.alert(`${pluginName}: Import from URL`, "Successfully imported GraphQL operations from URL");
      }
    } catch (error) {
      if (error instanceof Error) {
        console.log(`[ERROR] [${pluginName}: From URL] [${error}]`);
        context.app.alert("Error while importing GraphQL operations", error.message);
      }
    }
  }

} //*******
//******* Building or querying for GraphQL schema
//*******


exports.GraphQlGenDrivers = GraphQlGenDrivers;

async function buildGraphQLSchema(schemaString) {
  return new Promise(resolve => {
    let schemaParseResult = JSON.parse(schemaString)?.data;
    let schema = (0, _graphql.buildClientSchema)(schemaParseResult);
    resolve(schema);
  });
}

async function fetchGraphQLSchema(url, context) {
  let introspectionQuery = (0, _graphql.getIntrospectionQuery)();
  return new Promise(async (resolve, reject) => {
    let {
      request: httpRequest,
      Agent
    } = url.protocol === "https:" ? _https.default : _http.default;
    let request = httpRequest({
      method: "POST",
      protocol: url.protocol,
      host: url.host,
      path: url.pathname,
      pathname: url.pathname,
      headers: {
        Accept: "application/json",
        "Content-Type": "application/json"
      },
      agent: new Agent({
        rejectUnauthorized: false
      })
    }, response => {
      if (response.statusMessage === "OK" || response.statusCode?.toString().startsWith("2")) {
        let data = "";
        response.on("data", chunk => {
          data += chunk;
        });
        response.on("end", () => {
          let introspectionResult = JSON.parse(data)?.data;
          let schema = (0, _graphql.buildClientSchema)(introspectionResult);
          resolve(schema);
        });
      }

      response.on("error", reject);
    });
    var headers = await getSchemaRequestHeaders(context);
    headers.forEach(function (header) {
      request.setHeader(header.name, header.value);
    });
    request.on("error", reject);
    request.write(JSON.stringify({
      query: introspectionQuery
    }));
    request.end();
  });
}

async function getSchemaRequestHeaders(context) {
  const storedHeaders = await context.store.getItem('gql-gen:schemaRequestHeaders');

  try {
    console.log(storedHeaders);
    return JSON.parse(storedHeaders);
  } catch (e) {
    console.error("Loading schema headers failed: ", e);
    return [];
  }
} //*******
//******* Generating GraphQL operations from schema
//*******


async function getDefaultRequestHeaders(context) {
  const storedHeaders = await context.store.getItem('gql-gen:defaultRequestHeaders');

  try {
    console.log(storedHeaders);
    return JSON.parse(storedHeaders);
  } catch (e) {
    console.error("Loading request headers failed: ", e);
    return [];
  }
}

function mapFieldsToOperations(schema, fields, kind) {
  return fields.map(field => (0, _utils.buildOperationNodeForField)({
    schema,
    kind,
    field: field,
    depthLimit: 3,
    circularReferenceDepth: 2
  }));
}

async function generateOperations(schema) {
  let mutations = schema.getMutationType()?.getFields() || [];
  let queries = schema.getQueryType()?.getFields() || [];
  let subscriptions = schema.getSubscriptionType()?.getFields() || [];
  return {
    mutations: mapFieldsToOperations(schema, Object.keys(mutations), _graphql.OperationTypeNode.MUTATION),
    queries: mapFieldsToOperations(schema, Object.keys(queries), _graphql.OperationTypeNode.QUERY),
    subscriptions: mapFieldsToOperations(schema, Object.keys(subscriptions), _graphql.OperationTypeNode.SUBSCRIPTION)
  };
}
/**
* NOTE:
* Insomnia will generate new ids for any resource with an id that matches this regex: /__\w+_\d+__/g;
* https://github.com/Kong/insomnia/blob/develop/packages/insomnia-app/app/common/import.ts#L110
*/


function insomniaIdGenerator() {
  let index = 0;
  return function generateInsomniaId() {
    index += 1;
    return `__INSOMNIA_${index}__`;
  };
}

let generateInsomniaId = insomniaIdGenerator();

function getCurrentWorkspace(models) {
  let workspace = models.workspace;
  return workspace;
}

function mapVariablesToJson(variableNodes) {
  let variablesJson = new _tsJsonObject.JSONObject();
  variableNodes.forEach(element => {
    variablesJson[element.variable.name.value] = getVariableDefaultValue(element.type);
  });
  return JSON.stringify(variablesJson);
}

function getVariableDefaultValue(variableNode) {
  var typeName = "";

  if (variableNode.kind === _graphql.Kind.NON_NULL_TYPE && variableNode.type.kind === _graphql.Kind.NAMED_TYPE) {
    var innerTypeNode = variableNode.type;
    typeName = innerTypeNode.name.value;
  } else if (variableNode.kind === _graphql.Kind.NAMED_TYPE) {
    typeName = variableNode.name.value;
  }

  switch (typeName) {
    case 'Int':
      return 0;

    case 'Float':
      return 0;

    case 'Boolean':
      return false;

    default:
      return "";
  }
}

function getInsomniaResourcesFromOperations(operations, defaultHeaders, workspaceId, name, operationGroupName) {
  if (operations.length === 0) return [];
  let requestGroup = {
    parentId: workspaceId,
    name: operationGroupName,
    _type: "request_group",
    _id: generateInsomniaId()
  };

  function mapOperationToRequest(operation) {
    return {
      _id: generateInsomniaId(),
      _type: "request",
      body: {
        mimeType: "application/graphql",
        text: JSON.stringify({
          query: (0, _graphql.print)(operation),
          variables: mapVariablesToJson(operation.variableDefinitions)
        })
      },
      headers: [{
        name: "Content-Type",
        value: "application/json"
      }],
      name: operation.name?.value,
      method: "POST",
      url: name,
      parentId: requestGroup._id
    };
  }

  console.debug("Generating requests");
  let requests = operations.map(mapOperationToRequest);
  requests.forEach(request => {
    request.headers = request.headers.concat(defaultHeaders);
  });
  return [requestGroup, ...requests];
} //*******
//******* Import GraphQL operations into current workspace
//*******

/**
* Transforms the operations to requests and imports them to the current workspace.
*/


async function importToCurrentWorkspace(models, operations, baseUrl, folderName, context) {
  let workspace = getCurrentWorkspace(models);
  var defaultHeaders = await getDefaultRequestHeaders(context);
  var defaultHeaderObjects = defaultHeaders.map(mapHeaderToObject);
  let subscriptionsRequestGroup = getInsomniaResourcesFromOperations(operations.subscriptions, defaultHeaderObjects, workspace._id, baseUrl, folderName + "_Subscriptions");
  let queriesRequestGroup = getInsomniaResourcesFromOperations(operations.queries, defaultHeaderObjects, workspace._id, baseUrl, folderName + "_Queries");
  let mutationsRequestGroup = getInsomniaResourcesFromOperations(operations.mutations, defaultHeaderObjects, workspace._id, baseUrl, folderName + "_Mutations");

  function mapHeaderToObject(header) {
    return {
      name: header.name,
      value: header.value
    };
  }

  async function importResources(resources) {
    let insomniaExportLike = {
      resources,
      _type: "export",
      __export_format: 4
    };
    await context.data.import.raw(JSON.stringify(insomniaExportLike), {
      workspaceId: workspace._id
    });
  }

  let resources = [...subscriptionsRequestGroup, ...queriesRequestGroup, ...mutationsRequestGroup, workspace];
  let resourcesChunks = chunk(resources, 30);

  for (let resourcesChunk of resourcesChunks) {
    await importResources(resourcesChunk);
  }
}